﻿// Copyright (c) Dundas Data Visualization Inc. All rights reserved.
//
// THIS SOURCE CODE IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT ANY 
// WARRANTY. IT IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED 
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE 
// QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE 
// DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
//
// IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW THE AUTHOR WILL BE LIABLE TO YOU 
// FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES 
// ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT 
// LIMITED TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY 
// YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER 
// PROGRAMS), EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
// DAMAGES.

using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Net;
using System.Security.Principal;
using System.Text;
using System.Web;
using System.Web.Script.Serialization;

namespace NetViewerIntegrationSample
{
    public partial class Default : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            // Change this to switch from local logon (using hardcoded credentials) to Windows logon.
            bool isWindowsLogon = false;

            Guid logOnToken;

            if (isWindowsLogon)
            {
                // Windows logon.
                logOnToken = this.AuthenticateWindowsUser();
                if (logOnToken == Guid.Empty)
                {
                    // There is no Windows identity associated with the request - respond with a challenge (401 Unauthorized).
                    this.Response.StatusCode = (int)HttpStatusCode.Unauthorized;
                    return;
                }
            }
            else
            {
                // Local logon.
                logOnToken = this.AuthenticateLocalUser();
            }

            // Create the Dundas BI Element that will be picked up by the Dundas BI Embed library on load.
            string dundasBIElement =
                string.Format(
                        CultureInfo.CurrentCulture,
                        @"<div 
                            class=""dundas-bi-embed"" 
                            data-dundas-bi-url=""{0}"" 
                            data-controller-type=""Dashboard"" 
                            data-logon-token-id=""{1}"" 
                            data-file-system-id=""{2}"" 
                            data-view-options=""{3}"" >
                        </div>",
                        Constants.DundasBIServerUri,
                        logOnToken,
                        Constants.DundasBIDashboardId,
                        Constants.ViewOptions
                    );

            // Set the inside of the form to the Dundas BI element.
            form1.InnerHtml =
                dundasBIElement;

        }

        /// <summary>Authenticates with the DBI server for a local user.</summary>
        /// <returns>The logon token associated generated for a successful authentication.</returns>
        /// <exception cref="Exception">Authentication failed.</exception>
        private Guid AuthenticateLocalUser()
        {
            // The data represents a JSON-serialized Dundas.BI.WebApi.Models.LogOnTokenOptions object.
            var logOnTokenOptions =
                new
                {
                    accountName = Constants.DundasBIViewerUsername,
                    password = Constants.DundasBIViewerPassword,
                };

            return this.DoAuthenticate(logOnTokenOptions);
        }

        /// <summary>Authenticates with the DBI server for a local user.</summary>
        /// <returns>
        ///   The logon token associated generated for a successful authentication.
        ///   -or-
        ///   <see cref="Guid.Empty"/> if a Windows identity was not associated with the request.
        /// </returns>
        /// <exception cref="Exception">Authentication failed.</exception>
        private Guid AuthenticateWindowsUser()
        {
            // Ensure there is a Windows identity.
            var winIdent = HttpContext.Current.User.Identity as WindowsIdentity;
            if (winIdent == null || winIdent.IsAuthenticated == false)
            {
                return Guid.Empty;
            }

            // The data represents a JSON-serialized Dundas.BI.WebApi.Models.LogOnTokenOptions object.
            var logOnTokenOptions =
                new
                {
                    isWindowsLogOn = true,
                };

            using (WindowsImpersonationContext impersonationContext = winIdent.Impersonate())
            {
                return this.DoAuthenticate(logOnTokenOptions);
            }
        }

        /// <summary>Authenticates with the DBI server.</summary>
        /// <param name="logOnTokenOptions">The log on token options.</param>
        /// <returns>The logon token associated generated for a successful authentication.</returns>
        /// <exception cref="Exception">Authentication failed.</exception>
        private Guid DoAuthenticate(object logOnTokenOptions)
        {
            // Create the REST logon endpoint URI.
            UriBuilder uriBuilder = new UriBuilder(Constants.DundasBIServerUri);
            uriBuilder.Path = uriBuilder.Path.TrimEnd('/') + "/api/LogOn/Token/";
            Uri logonEndpoint = uriBuilder.Uri;

            // Create string used in request body.
            JavaScriptSerializer javaScriptSerializer = new JavaScriptSerializer();
            string logOnTokenOptionsAsString =
                javaScriptSerializer.Serialize(logOnTokenOptions);

            // Create the HttpWebRequest.
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(logonEndpoint);
            request.Method = "POST";
            request.UseDefaultCredentials = true;
            request.ContentType = "application/json";
            request.ContentLength = logOnTokenOptionsAsString.Length;

            // Write the request body to the stream.
            using (StreamWriter requestWriter = new StreamWriter(request.GetRequestStream(), System.Text.Encoding.ASCII))
            {
                requestWriter.Write(logOnTokenOptionsAsString);
            }

            // Send the request and get the response.
            string responseString;
            using (WebResponse webResponse = request.GetResponse())
            {
                Stream webStream = webResponse.GetResponseStream();
                using (StreamReader responseReader = new StreamReader(webStream))
                {
                    responseString = responseReader.ReadToEnd();
                }

                for (int i = 0; i < webResponse.Headers.Count; ++i)
                {
                    Console.WriteLine("\nHeader Name:{0}, Value :{1}", webResponse.Headers.Keys[i], webResponse.Headers[i]);
                }
            }

            // Process the response.
            Dictionary<string, object> responseData =
                (Dictionary<string, object>)javaScriptSerializer.Deserialize(responseString, typeof(object));

            string logOnFailureReason = responseData["logOnFailureReason"].ToString();
            string message = responseData["message"].ToString();

            if (logOnFailureReason != "None")
            {
                // Logon error.
                throw new Exception(logOnFailureReason + ": " + message);
            }

            // Authentication successful - return the logon token.
            Guid logOnToken = new Guid(responseData["logOnToken"].ToString());
            return logOnToken;
        }
    }
}
